@@ -42,7 +42,7 @@ openapi_schema = {
                         },
                     },
                 },
-                "summary": "Save Item No Body Post",
+                "summary": "Save Item No Body",
                 "operationId": "save_item_no_body_items__post",
                 "requestBody": {
                     "content": {
