@@ -60,7 +60,7 @@ openapi_schema = {
                         },
                     },
                 },
-                "summary": "Read Current User Post",
+                "summary": "Read Current User",
                 "operationId": "read_current_user_login_post",
                 "requestBody": {
                     "content": {
@@ -82,7 +82,7 @@ openapi_schema = {
                         "content": {"application/json": {"schema": {}}},
                     }
                 },
-                "summary": "Read Current User Get",
+                "summary": "Read Current User",
                 "operationId": "read_current_user_users_me_get",
                 "security": [{"OAuth2": []}],
             }
